"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildThemedComponentsInternal = exports.buildStyles = void 0;
// Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
// SPDX-License-Identifier: Apache-2.0
const style_1 = require("./tasks/style");
Object.defineProperty(exports, "buildStyles", { enumerable: true, get: function () { return style_1.buildStyles; } });
const preset_1 = require("./tasks/preset");
const internal_tokens_1 = require("./tasks/internal-tokens");
const public_tokens_1 = require("./tasks/public-tokens");
const theme_1 = require("../shared/theme");
const inline_stylesheets_1 = require("./inline-stylesheets");
const properties_1 = require("./properties");
const needed_tokens_1 = __importDefault(require("./needed-tokens"));
/**
 * Builds themed components and optionally design tokens, if not skipped.
 *
 * The styles will be build with three inline stylesheets:
 * * `awsui:environment` - Stylesheet containing a simple environment context
 * * `awsui:globals` - Root stylesheet with custom property assignments
 * * `awsui:tokens`  - Mapping between SASS variables and var() assignments
 *
 * If designTokensOuputDir is specified and not skipped, three with designTokensFileName will be generated:
 * * Typescript
 * * Typescript definitions
 * * SCSS
 *
 * @param params build themed components parameters
 */
async function buildThemedComponentsInternal(params) {
    const { primary, exposed, themeable, variablesMap, componentsOutputDir, designTokensOutputDir, scssDir, secondary = [], designTokensFileName = 'index', skip = [], descriptions = {}, jsonSchema = false, failOnDeprecations, } = params;
    if (!skip.includes('design-tokens') && !designTokensOutputDir) {
        throw new Error('designTokensOutputDir needs to be specified if not skipped');
    }
    const neededTokens = (0, needed_tokens_1.default)(scssDir, variablesMap, exposed);
    const resolution = (0, theme_1.resolveTheme)(primary);
    const defaults = (0, theme_1.reduce)(resolution, primary, (0, theme_1.defaultsReducer)());
    const propertiesMap = (0, properties_1.calculatePropertiesMap)([primary, ...secondary], variablesMap);
    const styleTask = (0, style_1.buildStyles)(scssDir, componentsOutputDir, (0, inline_stylesheets_1.getInlineStylesheets)(primary, secondary, defaults, variablesMap, propertiesMap, neededTokens), { failOnDeprecations });
    const internalTokensTask = (0, internal_tokens_1.createInternalTokenFiles)(defaults, propertiesMap, componentsOutputDir);
    const preset = {
        theme: primary,
        secondary,
        themeable,
        exposed,
        variablesMap,
        propertiesMap,
    };
    const presetTask = skip.includes('preset') ? Promise.resolve() : (0, preset_1.createPresetFiles)(preset, componentsOutputDir);
    const designTokensTask = skip.includes('design-tokens') || !designTokensOutputDir
        ? Promise.resolve()
        : (0, public_tokens_1.createPublicTokenFiles)({
            preset,
            resolution: defaults,
            outputDir: designTokensOutputDir,
            fileName: designTokensFileName,
            descriptions,
            jsonSchema,
        });
    await Promise.all([internalTokensTask, designTokensTask, presetTask, styleTask]);
}
exports.buildThemedComponentsInternal = buildThemedComponentsInternal;
