"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.validateJson = exports.getThemeJSONSchema = void 0;
// Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
// SPDX-License-Identifier: Apache-2.0
const jsonschema_1 = require("jsonschema");
const stringValueSchema = { type: 'string' };
const colorValueSchema = {
    type: 'string',
    pattern: '#[0-9a-f]{6}|rgba\\(\\d{1,3}%?(,\\s?\\d{1,3}%?){2},\\s?(1|0|0?\\.\\d+)\\)|transparent',
};
const spaceValueSchema = {
    type: 'string',
    pattern: '\\d+(px|rem|%)',
};
const borderWidthValueSchema = {
    type: 'string',
    pattern: '\\d+(px|rem|em)',
};
const textSizeValueSchema = {
    type: 'string',
    pattern: '\\d+(px|rem|em)',
};
const textWeightValueSchema = {
    type: 'string',
    pattern: '300|400|700|900|normal|bold|light|heavy',
};
const durationValueSchema = { type: 'string', pattern: '\\d+m?s' };
const visualModes = ['light', 'dark'];
const densityModes = ['comfortable', 'compact'];
const motionModes = ['default', 'disabled'];
const getComplexValueSchema = (valueSchema, propertiesNames) => ({
    type: 'object',
    properties: propertiesNames.reduce((acc, current) => {
        acc[current] = valueSchema;
        return acc;
    }, {}),
    required: propertiesNames,
    additionalProperties: false,
});
const getTokenSchema = (valueSchema) => ({
    type: 'object',
    properties: {
        $value: valueSchema,
        $description: { type: 'string' },
    },
    required: ['$value'],
    additionalProperties: false,
});
const tokensSchema = {
    type: 'object',
    patternProperties: {
        '^color-': getTokenSchema(getComplexValueSchema(colorValueSchema, visualModes)),
        '^font-family-': getTokenSchema(stringValueSchema),
        '^border-radius-': getTokenSchema(spaceValueSchema),
        '^border-width-': getTokenSchema(borderWidthValueSchema),
        '^space-': getTokenSchema(getComplexValueSchema(spaceValueSchema, densityModes)),
        '^motion-duration-': getTokenSchema(getComplexValueSchema(durationValueSchema, motionModes)),
        '^motion-easing-': getTokenSchema(getComplexValueSchema(stringValueSchema, motionModes)),
        '^motion-keyframes-': getTokenSchema(getComplexValueSchema(stringValueSchema, motionModes)),
        '^shadow-': getTokenSchema(getComplexValueSchema(stringValueSchema, visualModes)),
        '^font-size-': getTokenSchema(textSizeValueSchema),
        '^line-height-': getTokenSchema(textSizeValueSchema),
        '^font-weight-': getTokenSchema(textWeightValueSchema),
        '^letter-spacing-': getTokenSchema(textSizeValueSchema),
    },
    additionalProperties: false,
};
function getThemeJSONSchema(theme) {
    const contextsNames = Object.keys(theme.contexts || {});
    return {
        $schema: 'https://json-schema.org/draft/2020-12/schema',
        type: 'object',
        properties: {
            tokens: tokensSchema,
            contexts: {
                type: 'object',
                properties: contextsNames.reduce((acc, current) => {
                    acc[current] = {
                        type: 'object',
                        properties: { tokens: tokensSchema },
                        required: ['tokens'],
                        additionalProperties: false,
                    };
                    return acc;
                }, {}),
                required: contextsNames,
                additionalProperties: false,
            },
        },
        required: ['tokens', 'contexts'],
        additionalProperties: false,
    };
}
exports.getThemeJSONSchema = getThemeJSONSchema;
function validateJson(themeJson, themeJsonSchema) {
    const validationResult = (0, jsonschema_1.validate)(themeJson, themeJsonSchema);
    if (validationResult.valid) {
        return true;
    }
    else {
        throw new Error(`Tokens validation error: ${validationResult.errors[0].stack}`);
    }
}
exports.validateJson = validateJson;
