"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MultiThemeCreator = void 0;
// Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
// SPDX-License-Identifier: Apache-2.0
const selector_1 = require("../styles/selector");
const theme_1 = require("../theme");
const abstract_1 = require("./abstract");
const single_1 = require("./single");
const stylesheet_1 = __importDefault(require("./stylesheet"));
const utils_1 = require("./utils");
/**
 * Extends the single theme stylesheet creator by a secondary theme, which takes the existing theme as
 * base.
 */
class MultiThemeCreator extends abstract_1.AbstractCreator {
    constructor(themes, ruleCreator) {
        super();
        this.themes = themes;
        this.ruleCreator = ruleCreator;
    }
    create() {
        const globalThemes = this.themes.filter((theme) => (0, selector_1.isGlobalSelector)(theme.selector));
        if (globalThemes.length > 1) {
            throw new Error(`Themes ${globalThemes
                .map(({ id }) => id)
                .join(', ')} have a global selector. It is not supported to have more than one global theme. It produces unpredictable styling results.`);
        }
        if (!globalThemes.length) {
            // If there is no root theme, all themes are scoped by their root selector. No interference.
            const stylesheets = this.themes.map((theme) => new single_1.SingleThemeCreator(theme, this.ruleCreator).create());
            const result = new stylesheet_1.default();
            stylesheets.forEach((stylesheet) => {
                stylesheet.getAllRules().map((rule) => result.appendRuleWithPath(rule, stylesheet.getPath(rule) ?? []));
            });
            return result;
        }
        const [globalTheme] = globalThemes;
        const stylesheet = new single_1.SingleThemeCreator(globalTheme, this.ruleCreator).create();
        const secondaries = this.getThemesWithout(globalTheme);
        secondaries.forEach((secondary) => {
            this.appendRulesForSecondary(stylesheet, globalTheme, secondary);
        });
        return stylesheet;
    }
    appendRulesForSecondary(stylesheet, primary, secondary) {
        const secondaryResolution = (0, theme_1.resolveTheme)(secondary);
        const defaults = (0, theme_1.reduce)(secondaryResolution, secondary, (0, theme_1.defaultsReducer)());
        const rootRule = this.ruleCreator.create({ global: [secondary.selector] }, defaults);
        const parentRule = this.findRule(stylesheet, { global: [primary.selector] });
        MultiThemeCreator.appendRuleToStylesheet(stylesheet, rootRule, (0, utils_1.compact)([parentRule]));
        MultiThemeCreator.forEachOptionalModeState(secondary, (mode, state) => {
            const optionalState = mode.states[state];
            const modeResolution = (0, theme_1.reduce)(secondaryResolution, secondary, (0, theme_1.modeReducer)(mode, state));
            const modeRule = this.ruleCreator.create({ global: [secondary.selector, optionalState.selector], media: optionalState.media }, modeResolution);
            const parentModeRule = stylesheet.findRule(this.ruleCreator.selectorFor({
                global: [primary.selector, optionalState.selector],
            }));
            MultiThemeCreator.appendRuleToStylesheet(stylesheet, modeRule, (0, utils_1.compact)([rootRule, parentModeRule, parentRule]));
        });
        MultiThemeCreator.forEachContext(secondary, (context) => {
            const contextResolution = (0, theme_1.reduce)((0, theme_1.resolveContext)(secondary, context), secondary, (0, theme_1.defaultsReducer)());
            const contextRule = this.ruleCreator.create({ global: [secondary.selector], local: [context.selector] }, contextResolution);
            const parentContextRule = stylesheet.findRule(this.ruleCreator.selectorFor({
                global: [primary.selector],
                local: [context.selector],
            }));
            MultiThemeCreator.appendRuleToStylesheet(stylesheet, contextRule, (0, utils_1.compact)([parentContextRule, rootRule, parentRule]));
            const contextRuleGlobal = this.ruleCreator.create({ global: [secondary.selector, context.selector] }, contextResolution);
            MultiThemeCreator.appendRuleToStylesheet(stylesheet, contextRuleGlobal, (0, utils_1.compact)([rootRule, parentContextRule, parentRule]));
        });
        MultiThemeCreator.forEachContextWithinOptionalModeState(secondary, (context, mode, state) => {
            const optionalState = mode.states[state];
            const contextResolution = (0, theme_1.reduce)((0, theme_1.resolveContext)(secondary, context), secondary, (0, theme_1.modeReducer)(mode, state));
            const contextRule = this.findRule(stylesheet, { global: [secondary.selector], local: [context.selector] });
            const modeRule = this.findRule(stylesheet, {
                global: [secondary.selector, optionalState.selector],
            });
            const contextAndModeRule = this.ruleCreator.create({
                global: [secondary.selector, optionalState.selector],
                local: [context.selector],
                media: optionalState.media,
            }, contextResolution);
            const parentContextRule = stylesheet.findRule(this.ruleCreator.selectorFor({ global: [primary.selector], local: [context.selector] }));
            const parentModeRule = stylesheet.findRule(this.ruleCreator.selectorFor({
                global: [primary.selector, optionalState.selector],
            }));
            const parentContextAndModeRule = stylesheet.findRule(this.ruleCreator.selectorFor({
                global: [primary.selector, optionalState.selector],
                local: [context.selector],
            }));
            MultiThemeCreator.appendRuleToStylesheet(stylesheet, contextAndModeRule, (0, utils_1.compact)([
                contextRule,
                parentContextAndModeRule,
                parentContextRule,
                modeRule,
                rootRule,
                parentModeRule,
                parentRule,
            ]));
            const parentContextAndModeRuleGlobal = stylesheet.findRule(this.ruleCreator.selectorFor({ global: [secondary.selector, context.selector] }));
            const contextAndModeRuleGlobal = this.ruleCreator.create({
                global: [secondary.selector, optionalState.selector, context.selector],
                media: optionalState.media,
            }, contextResolution);
            MultiThemeCreator.appendRuleToStylesheet(stylesheet, contextAndModeRuleGlobal, (0, utils_1.compact)([
                contextRule,
                modeRule,
                parentContextAndModeRuleGlobal,
                rootRule,
                parentContextRule,
                parentModeRule,
                parentRule,
            ]));
        });
        return stylesheet;
    }
    findRule(stylesheet, config) {
        const rule = stylesheet.findRule(this.ruleCreator.selectorFor(config));
        if (!rule) {
            // Rules are creating in reverse order to dependency, which is why
            // we should always have a rule.
            throw new Error(`No rule for selector ${JSON.stringify(config)} found`);
        }
        return rule;
    }
    getThemesWithout(theme) {
        const idx = this.themes.indexOf(theme);
        return [...this.themes.slice(0, idx), ...this.themes.slice(idx + 1)];
    }
}
exports.MultiThemeCreator = MultiThemeCreator;
