"use strict";
// Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
// SPDX-License-Identifier: Apache-2.0
Object.defineProperty(exports, "__esModule", { value: true });
exports.Declaration = exports.Rule = void 0;
/**
 * Allows building a stylesheet by appending selectors and declarations. A call of
 * `toString` will return CSS.
 */
class Stylesheet {
    constructor() {
        this.rulesMap = new Map();
        this.paths = new Map();
        this.counter = 0;
    }
    appendRule(rule) {
        this.rulesMap.set(rule.selector, [rule, this.counter++]);
    }
    appendRuleWithPath(rule, path) {
        this.rulesMap.set(rule.selector, [rule, this.counter++]);
        this.paths.set(rule, path);
    }
    removeRule(rule) {
        this.rulesMap.delete(rule.selector);
        this.paths.delete(rule);
    }
    findRule(selector) {
        const ruleOrUndefined = this.rulesMap.get(selector);
        return ruleOrUndefined?.[0];
    }
    getPath(rule) {
        const path = this.paths.get(rule);
        if (!path) {
            throw new Error(`No path for rule with selector: ${rule.selector}`);
        }
        return path;
    }
    getAllRules() {
        const rules = [];
        this.paths.forEach((_, key) => rules.push(key));
        return rules;
    }
    /**
     * @returns CSS
     */
    toString() {
        return asValuesArray(this.rulesMap)
            .map((rule) => rule.toString())
            .join('\n');
    }
}
exports.default = Stylesheet;
class Rule {
    constructor(selector, media) {
        this.declarationsMap = new Map();
        this.counter = 0;
        this.selector = selector;
        this.media = media;
    }
    appendDeclaration(declaration) {
        this.declarationsMap.set(declaration.property, [declaration, this.counter++]);
    }
    clear() {
        this.declarationsMap = new Map();
        this.counter = 0;
    }
    getAllDeclarations() {
        return asValuesArray(this.declarationsMap);
    }
    size() {
        return this.declarationsMap.size;
    }
    toString() {
        const lines = asValuesArray(this.declarationsMap).map((decl) => decl.toString());
        const rule = `${this.selector}{\n\t${lines.join('\n\t')}\n}`;
        if (this.media) {
            return `@media ${this.media} {${rule}}`;
        }
        return rule;
    }
}
exports.Rule = Rule;
class Declaration {
    constructor(property, value) {
        this.property = property;
        this.value = value;
    }
    toString() {
        return `${this.property}:${this.value};`;
    }
}
exports.Declaration = Declaration;
function asValuesArray(map) {
    const tmp = [];
    map.forEach(([item, position]) => tmp.push([item, position]));
    tmp.sort(([itemA, posA], [itemB, posB]) => posA - posB);
    return tmp.map(([item]) => item);
}
