"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isSpecificTokenResolution = exports.isModeTokenResolution = exports.difference = exports.modeReducer = exports.defaultsReducer = exports.reduce = exports.resolveContext = exports.resolveThemeWithPaths = exports.resolveTheme = void 0;
const utils_1 = require("../utils");
const utils_2 = require("./utils");
/**
 * If a base theme is provided, only keep tokens that are in the override theme or those that
 * have an overridden token in their resolution path
 */
function resolveTheme(theme, baseTheme) {
    return resolveThemeWithPaths(theme, baseTheme).resolvedTheme;
}
exports.resolveTheme = resolveTheme;
function resolveThemeWithPaths(theme, baseTheme) {
    const resolvedTheme = {};
    const resolutionPaths = {};
    Object.keys(baseTheme?.tokens ?? theme.tokens).forEach((token) => {
        const mode = (0, utils_2.getMode)(baseTheme ?? theme, token);
        if (mode) {
            const modeTokenResolutionPaths = {};
            const resolvedToken = Object.keys(mode.states).reduce((acc, state) => {
                modeTokenResolutionPaths[state] = [];
                acc[state] = resolveToken(theme, token, modeTokenResolutionPaths[state], state, baseTheme);
                return acc;
            }, {});
            const tokenResolutionPathContainsOverriddenTokens = (0, utils_1.values)(modeTokenResolutionPaths).some((tokenResolutionPath) => tokenResolutionPath.some((pathToken) => pathToken in theme.tokens));
            if (!baseTheme || tokenResolutionPathContainsOverriddenTokens) {
                resolutionPaths[token] = modeTokenResolutionPaths;
                resolvedTheme[token] = resolvedToken;
            }
        }
        else {
            const tokenResolutionPath = [];
            const resolvedToken = resolveToken(theme, token, tokenResolutionPath, undefined, baseTheme);
            if (!baseTheme || tokenResolutionPath.some((pathToken) => pathToken in theme.tokens)) {
                resolutionPaths[token] = tokenResolutionPath;
                resolvedTheme[token] = resolvedToken;
            }
        }
    });
    return { resolvedTheme, resolutionPaths };
}
exports.resolveThemeWithPaths = resolveThemeWithPaths;
function resolveToken(theme, token, path, state, baseTheme) {
    if (!theme.tokens[token] && !baseTheme?.tokens[token]) {
        throw new Error(`Token ${token} does not exist in the theme.`);
    }
    if (path.indexOf(token) !== -1) {
        throw new Error(`Token ${token} has a circular dependency.`);
    }
    path.push(token);
    let assignment = theme.tokens[token] || baseTheme?.tokens[token];
    if (!assignment) {
        throw new Error(`Empty assignment for token ${token}`);
    }
    if ((0, utils_2.isModeValue)(assignment)) {
        if (!state) {
            throw new Error(`Mode resolution for token ${token} does not have any mode value. modes: ${JSON.stringify(assignment)}`);
        }
        assignment = assignment[state];
    }
    if ((0, utils_2.isReference)(assignment)) {
        const ref = (0, utils_2.getReference)(assignment);
        return resolveToken(theme, ref, path, state, baseTheme);
    }
    else {
        return assignment;
    }
}
function resolveContext(theme, context, baseTheme, themeResolution) {
    const tmp = (0, utils_1.cloneDeep)(theme);
    if (!baseTheme || !themeResolution) {
        tmp.tokens = {
            ...tmp.tokens,
            ...context.tokens,
        };
        return resolveTheme(tmp, baseTheme);
    }
    /**
     * The precedence of context tokens as specified by the API from highest to lowest is:
     * [override theme context] > [base theme context] > [override theme] [base theme].
     *
     * The precedence of tokens as defined in the generated CSS follows this order.
     * However, tokens that are declared in both the base theme and base theme
     * context and share the same value are only included in the base theme css. This
     * results in override theme tokens incorrectly taking precedence over base theme
     * context.
     *
     * To counteract this we can re-baseline the override context using all keys used
     * in the override theme with their respective values from the base theme context
     */
    const baseContext = baseTheme.contexts[context.id];
    tmp.tokens = {
        ...Object.keys(themeResolution).reduce((acc, key) => {
            const shouldSkipReset = (!(key in baseContext.tokens) && !(key in theme.tokens)) ||
                (0, utils_2.areAssignmentsEqual)(baseContext.tokens[key], theme.tokens[key] ?? baseTheme.tokens[key] // resolved key may not be in override theme
                );
            return shouldSkipReset
                ? acc
                : {
                    ...acc,
                    [key]: baseContext.tokens[key] ?? theme.tokens[key] ?? baseTheme.tokens[key],
                };
        }, {}),
        ...context.tokens,
    };
    return resolveTheme(tmp, baseTheme);
}
exports.resolveContext = resolveContext;
function reduce(resolution, theme, reducer, baseTheme) {
    return Object.keys(resolution).reduce((acc, token) => {
        const reduced = reducer(resolution[token], token, theme, baseTheme);
        if (reduced) {
            acc[token] = reduced;
        }
        return acc;
    }, {});
}
exports.reduce = reduce;
const defaultsReducer = () => (tokenResolution, token, theme, baseTheme) => {
    const mode = (0, utils_2.getMode)(baseTheme ?? theme, token);
    if (mode && isModeTokenResolution(tokenResolution)) {
        const defaultState = (0, utils_2.getDefaultState)(mode);
        return tokenResolution[defaultState];
    }
    else if (isSpecificTokenResolution(tokenResolution)) {
        return tokenResolution;
    }
    else {
        throw new Error(`Mismatch between resolution ${JSON.stringify(tokenResolution)} and mode ${mode}`);
    }
};
exports.defaultsReducer = defaultsReducer;
const modeReducer = (mode, state) => (tokenResolution, token, theme, baseTheme) => {
    const tokenMode = (0, utils_2.getMode)(baseTheme ?? theme, token);
    if (tokenMode && tokenMode.id === mode.id && isModeTokenResolution(tokenResolution)) {
        return tokenResolution[state];
    }
    else if (isSpecificTokenResolution(tokenResolution)) {
        return tokenResolution;
    }
};
exports.modeReducer = modeReducer;
function difference(base, other) {
    const result = {};
    Object.keys(other).forEach((token) => {
        const baseVal = base[token];
        const otherVal = other[token];
        if (isSpecificTokenResolution(baseVal) && isSpecificTokenResolution(otherVal) && baseVal !== otherVal) {
            result[token] = otherVal;
        }
        else if (isModeTokenResolution(baseVal) && isModeTokenResolution(otherVal)) {
            const resolved = Object.keys(otherVal).reduce((acc, state) => {
                if (baseVal[state] !== otherVal[state]) {
                    acc[state] = otherVal[state];
                }
                return acc;
            }, {});
            if (!isEmpty(resolved)) {
                result[token] = resolved;
            }
        }
    });
    return result;
}
exports.difference = difference;
function isModeTokenResolution(val) {
    return typeof val === 'object';
}
exports.isModeTokenResolution = isModeTokenResolution;
function isSpecificTokenResolution(val) {
    return typeof val === 'string';
}
exports.isSpecificTokenResolution = isSpecificTokenResolution;
const isEmpty = (obj) => Object.keys(obj).length === 0;
