"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getThemeFromPreset = exports.getContexts = exports.validateOverride = void 0;
// Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
// SPDX-License-Identifier: Apache-2.0
const utils_1 = require("../utils");
/**
 * This function compares the theme override against the list of tokens that are allowed
 * to be themed. Only those overrides that use allowed tokens are returned. Other tokens are
 * dropped and a warning is emitted to the console.
 * If the theme override contains visual-context-specific overrides, only those overrides
 * that specify known context IDs are returned. Other visual-context-specific overrides
 * are dropped and a warning is emitted to the console.
 */
function validateOverride(override, themeable, availableContexts) {
    // The following two validations should not be possible, if customer uses
    // Typescript in their project. However, it might be easily overseen in a Javascript
    // project. Therefore, we supply additional hints beyond just types.
    if (typeof override.tokens !== 'object' || Array.isArray(override.tokens) || override.tokens === null) {
        throw new Error(`Missing required "tokens" object field in ${JSON.stringify(override)}`);
    }
    // This cache is used so that we emit only one warning per token name.
    const unthemeableTokenWarningCache = {};
    function isThemeable(token) {
        const isThemeable = (0, utils_1.includes)(themeable, token);
        if (!isThemeable && !(token in unthemeableTokenWarningCache)) {
            console.warn(`${token} is not themeable and will be ignored during theming`);
            unthemeableTokenWarningCache[token] = true;
        }
        return isThemeable;
    }
    function isValidContextId(contextId) {
        const isValid = (0, utils_1.includes)(availableContexts, contextId);
        if (!isValid) {
            console.warn(`${contextId} is not a valid ID of a visual context and will be ignored during theming.`);
        }
        return isValid;
    }
    const tokensEntries = (0, utils_1.entries)(override.tokens).filter(([token]) => isThemeable(token));
    const contextEntries = (override.contexts
        ? (0, utils_1.entries)(override.contexts).filter(([, context]) => context !== undefined)
        : [])
        .filter(([contextId]) => isValidContextId(contextId))
        .map(([contextId, context]) => {
        const filteredTokens = (0, utils_1.entries)(context.tokens).filter(([token]) => isThemeable(token));
        const from = (0, utils_1.fromEntries)(filteredTokens);
        const newContext = {
            ...context,
            tokens: from,
        };
        return [contextId, newContext];
    });
    return {
        contexts: (0, utils_1.fromEntries)(contextEntries),
        tokens: (0, utils_1.fromEntries)(tokensEntries),
    };
}
exports.validateOverride = validateOverride;
function getContexts(preset) {
    const themes = [preset.theme, ...(preset.secondary ?? [])];
    const contexts = [];
    for (const theme of themes) {
        Object.keys(theme.contexts).forEach((contextName) => {
            if (contexts.indexOf(contextName) === -1) {
                contexts.push(contextName);
            }
        });
    }
    return contexts;
}
exports.getContexts = getContexts;
function getThemeFromPreset(preset, baseThemeId) {
    if (!baseThemeId) {
        return preset.theme;
    }
    const themesMap = [preset.theme, ...(preset.secondary ?? [])].reduce((accThemesMap, currentTheme) => {
        accThemesMap[currentTheme.id] = currentTheme;
        return accThemesMap;
    }, {});
    if (!themesMap[baseThemeId]) {
        throw new Error(`Specified baseThemeId '${baseThemeId}' is not available. Available values are ${Object.keys(themesMap)
            .map((value) => `'${value}'`)
            .join(', ')}.`);
    }
    return themesMap[baseThemeId];
}
exports.getThemeFromPreset = getThemeFromPreset;
